@@ -1,4 +1,4 @@
-/* Copyright (C) 2017-2019 Free Software Foundation, Inc.
+/* Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,8 +15,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef ARCH_TIC6X_H
-#define ARCH_TIC6X_H
+#ifndef GDB_ARCH_TIC6X_H
+#define GDB_ARCH_TIC6X_H
 
 enum c6x_feature
 {
@@ -28,4 +28,4 @@ enum c6x_feature
 
 target_desc *tic6x_create_target_description (enum c6x_feature feature);
 
-#endif /* ARCH_TIC6X_H */
+#endif /* GDB_ARCH_TIC6X_H */
