@@ -1,6 +1,6 @@
 /* FreeBSD/riscv target support, prototypes.
 
-   Copyright (C) 2018-2023 Free Software Foundation, Inc.
+   Copyright (C) 2018-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,8 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef RISCV_FBSD_TDEP_H
-#define RISCV_FBSD_TDEP_H
+#ifndef GDB_RISCV_FBSD_TDEP_H
+#define GDB_RISCV_FBSD_TDEP_H
 
 #include "regset.h"
 
@@ -35,4 +35,4 @@
 extern const struct regset riscv_fbsd_gregset;
 extern const struct regset riscv_fbsd_fpregset;
 
-#endif /* RISCV_FBSD_TDEP_H */
+#endif /* GDB_RISCV_FBSD_TDEP_H */
