@@ -1,6 +1,6 @@
 /* Code dealing with register stack frames, for GDB, the GNU debugger.
 
-   Copyright (C) 2003-2020 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,8 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#if !defined (SENTINEL_FRAME_H)
-#define SENTINEL_FRAME_H 1
+#ifndef GDB_SENTINEL_FRAME_H
+#define GDB_SENTINEL_FRAME_H
 
 struct frame_unwind;
 struct regcache;
@@ -36,4 +36,4 @@ extern void *sentinel_frame_cache (struct regcache *regcache);
 
 extern const struct frame_unwind sentinel_frame_unwind;
 
-#endif /* !defined (SENTINEL_FRAME_H)  */
+#endif /* GDB_SENTINEL_FRAME_H */
