@@ -1,6 +1,6 @@
 /* Native-dependent code for OpenBSD/sparc64.
 
-   Copyright (C) 2003-2019 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "gdbcore.h"
 #include "regcache.h"
 #include "target.h"
@@ -109,8 +108,9 @@ sparc64obsd_supply_pcb (struct regcache *regcache, struct pcb *pcb)
 /* Add some extra features to the generic SPARC target.  */
 static sparc_target<obsd_nat_target> the_sparc64_obsd_nat_target;
 
+void _initialize_sparc64obsd_nat ();
 void
-_initialize_sparc64obsd_nat (void)
+_initialize_sparc64obsd_nat ()
 {
   sparc_supply_gregset = sparc64_supply_gregset;
   sparc_collect_gregset = sparc64_collect_gregset;
