@@ -1,5 +1,5 @@
 /* Pthreads test program.
-   Copyright 1996-2015 Free Software Foundation, Inc.
+   Copyright 1996-2024 Free Software Foundation, Inc.
 
    Written by Fred Fish of Cygnus Support
    Contributed by Cygnus Support
@@ -23,25 +23,13 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <unistd.h>
-
-/* Under HPUX 10, the second arg of pthread_create
-   is prototyped to be just a "pthread_attr_t", while under Solaris it
-   is a "pthread_attr_t *".  Arg! */
-
-#if defined (__hpux__)
-#define PTHREAD_CREATE_ARG2(arg) arg
-#define PTHREAD_CREATE_NULL_ARG2 null_attr
-static pthread_attr_t null_attr;
-#else
-#define PTHREAD_CREATE_ARG2(arg) &arg
-#define PTHREAD_CREATE_NULL_ARG2 NULL
-#endif
+#include <string.h>
+#include <errno.h>
 
 static int verbose = 0;
 
 static void
-common_routine (arg)
-     int arg;
+common_routine (int arg)
 {
   static int from_thread1;
   static int from_thread2;
@@ -49,7 +37,8 @@ common_routine (arg)
   static int hits;
   static int full_coverage;
 
-  if (verbose) printf("common_routine (%d)\n", arg);
+  if (verbose)
+    printf ("common_routine (%d)\n", arg);
   hits++;
   switch (arg)
     {
@@ -73,13 +62,15 @@ thread1 (void *arg)
   int i;
   int z = 0;
 
-  if (verbose) printf ("thread1 (%0lx) ; pid = %d\n", (long) arg, getpid ());
-  for (i=1; i <= 10000000; i++)
+  if (verbose)
+    printf ("thread1 (%0lx) ; pid = %d\n", (long) arg, getpid ());
+  for (i = 1; i <= 10000000; i++)
     {
-      if (verbose) printf("thread1 %ld\n", (long) pthread_self ());
+      if (verbose)
+	printf ("thread1 %ld\n", (long) pthread_self ());
       z += i;
       common_routine (1);
-      sleep(1);
+      sleep (1);
     }
   return (void *) 0;
 }
@@ -90,81 +81,97 @@ thread2 (void * arg)
   int i;
   int k = 0;
 
-  if (verbose) printf ("thread2 (%0lx) ; pid = %d\n", (long) arg, getpid ());
-  for (i=1; i <= 10000000; i++)
+  if (verbose)
+    printf ("thread2 (%0lx) ; pid = %d\n", (long) arg, getpid ());
+  for (i = 1; i <= 10000000; i++)
     {
-      if (verbose) printf("thread2 %ld\n", (long) pthread_self ());
+      if (verbose)
+	printf ("thread2 %ld\n", (long) pthread_self ());
       k += i;
       common_routine (2);
-      sleep(1);
+      sleep (1);
     }
-  sleep(100);
+  sleep (100);
   return (void *) 0;
 }
 
 void
-foo (a, b, c)
-     int a, b, c;
+foo (int a, int b, int c)
 {
   int d, e, f;
 
-  if (verbose) printf("a=%d\n", a);
+  if (verbose)
+    printf ("a=%d\n", a);
+}
+
+/* Similar to perror, but use ERR instead of errno.  */
+
+static void
+print_error (const char *ctx, int err)
+{
+  fprintf (stderr, "%s: %s (%d)\n", ctx, strerror (err), err);
 }
 
 int
-main(argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   pthread_t tid1, tid2;
   int j;
   int t = 0;
   void (*xxx) ();
   pthread_attr_t attr;
+  int res;
 
-  if (verbose) printf ("pid = %d\n", getpid());
+  if (verbose)
+    printf ("pid = %d\n", getpid ());
 
   foo (1, 2, 3);
 
-  if (pthread_attr_init (&attr))
+  res = pthread_attr_init (&attr);
+  if (res != 0)
     {
-      perror ("pthread_attr_init 1");
+      print_error ("pthread_attr_init 1", res);
       exit (1);
     }
 
 #ifdef PTHREAD_SCOPE_SYSTEM
-  if (pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM))
+  res = pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM);
+  if (res != 0 && res != ENOTSUP)
     {
-      perror ("pthread_attr_setscope 1");
+      print_error ("pthread_attr_setscope 1", res);
       exit (1);
     }
 #endif
 
-  if (pthread_create (&tid1, PTHREAD_CREATE_ARG2(attr), thread1, (void *) 0xfeedface))
+  res = pthread_create (&tid1, &attr, thread1, (void *) 0xfeedface);
+  if (res != 0)
     {
-      perror ("pthread_create 1");
+      print_error ("pthread_create 1", res);
       exit (1);
     }
-  if (verbose) printf ("Made thread %ld\n", (long) tid1);
+  if (verbose)
+    printf ("Made thread %ld\n", (long) tid1);
   sleep (1);
 
-  if (pthread_create (&tid2, PTHREAD_CREATE_NULL_ARG2, thread2, (void *) 0xdeadbeef))
+  res = pthread_create (&tid2, NULL, thread2, (void *) 0xdeadbeef);
+  if (res != 0)
     {
-      perror ("pthread_create 2");
+      print_error ("pthread_create 2", res);
       exit (1);
     }
-  if (verbose) printf("Made thread %ld\n", (long) tid2);
+  if (verbose)
+    printf ("Made thread %ld\n", (long) tid2);
 
   sleep (1);
 
   for (j = 1; j <= 10000000; j++)
     {
-      if (verbose) printf("top %ld\n", (long) pthread_self ());
+      if (verbose)
+	printf ("top %ld\n", (long) pthread_self ());
       common_routine (0);
-      sleep(1);
+      sleep (1);
       t += j;
     }
   
-  exit(0);
+  exit (0);
 }
-
