@@ -1,7 +1,7 @@
 /*  Lattice Mico32 UART model.
     Contributed by Jon Beniston <jon@beniston.com>
     
-   Copyright (C) 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2009-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -18,11 +18,15 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+/* This must come before any other includes.  */
+#include "defs.h"
+
 #include "sim-main.h"
 #include "hw-main.h"
 #include "sim-assert.h"
 
 #include <stdio.h>
+#include <sys/select.h>
 #include <sys/time.h>
 
 struct lm32uart
@@ -93,7 +97,7 @@ do_uart_tx_event (struct hw *me, void *data)
       hw_port_event (me, INT_PORT, 1);
     }
 
-  /* Indicate which interrupt has occured.  */
+  /* Indicate which interrupt has occurred.  */
   uart->iir = MICOUART_IIR_TXRDY;
 
   /* Indicate THR is empty.  */
@@ -288,7 +292,6 @@ static void
 lm32uart_finish (struct hw *me)
 {
   struct lm32uart *uart;
-  int i;
 
   uart = HW_ZALLOC (me, struct lm32uart);
   set_hw_data (me, uart);
